//=================================================================================================
// Samplers
//=================================================================================================

Texture2D u_MaterialDiffuseTexture;
SamplerState u_MaterialDiffuseTextureSampler;

//=================================================================================================
// Constant buffers
//=================================================================================================


#if defined(_WP8) || defined(_WINRT_ARM)
#define LIGHTCOUNT 1
#else
#define LIGHTCOUNT 8
#endif

cbuffer cbGlobal
{
	float u_ActiveLights[LIGHTCOUNT];
	float4 u_LightPositions[LIGHTCOUNT];
	float4 u_LightDiffuseColors[LIGHTCOUNT];
	float4 u_LightSpecularColors[LIGHTCOUNT];
	float4 u_GlobalAmbientColor;
	matrix u_ViewMatrixInverse;
};

cbuffer cbMaterial
{
	float u_LightingEnabled;
	float u_VertexColorEnabled;
	float u_AlphaTestEnabled;
	float u_AlphaTestThreshold;
	float4 u_MaterialAmbientColor;
	float4 u_MaterialDiffuseColor;
	float4 u_MaterialSpecularColor;
	float u_MaterialShininess;
	float u_MaterialHasDiffuseTexture;
	matrix u_TextureTransformMatrix;
};

cbuffer cbObject
{
	matrix u_WorldViewProjectionMatrix;
	matrix u_WorldMatrix;
	matrix u_WorldMatrixInverseTranspose;
};

//=================================================================================================
// Structures
//=================================================================================================

struct VS_IN
{
    float3 Position : POSITION;
    float3 Normal   : NORMAL;
	float2 TexCoord : TEXCOORD;
    float4 Color    : COLOR;
};

struct PS_IN
{
    float4 Position : SV_POSITION;
	float4 WorldPos : WORLDPOS;
	float4 Normal   : NORMAL;
	float2 TexCoord : TEXCOORD;
    float4 Color    : COLOR;
};

//=================================================================================================
// Vertex shader
//=================================================================================================

PS_IN vs_main(VS_IN In)
{
    PS_IN Out;
    Out.Position = mul(u_WorldViewProjectionMatrix, float4(In.Position, 1.0f));
	Out.WorldPos = mul(u_WorldMatrix, float4(In.Position, 1.0f));
	Out.Normal = mul(u_WorldMatrixInverseTranspose, float4(In.Normal, 0.0f));
	Out.TexCoord = mul(u_TextureTransformMatrix, float4(In.TexCoord, 0.0f, 1.0f)).xy;
    Out.Color = In.Color;
    return Out;
}

//=================================================================================================
// Pixel shader
//=================================================================================================

float4 ps_main(PS_IN In) : SV_TARGET
{
	float4 result;
	
    if(u_LightingEnabled)
    {
		float4 matAmbientColor;
		float4 matDiffuseColor;
		float4 matSpecularColor;
		float matShininess;
		
		if(u_VertexColorEnabled)
		{
			matAmbientColor = In.Color;
			matDiffuseColor = In.Color;
			matSpecularColor = u_MaterialSpecularColor;
			matShininess = u_MaterialShininess;
		}
		else
		{
			matAmbientColor = u_MaterialAmbientColor;
			matDiffuseColor = u_MaterialDiffuseColor;
			matSpecularColor = u_MaterialSpecularColor;
			matShininess = u_MaterialShininess;
		}
		
        float3 color = matAmbientColor.rgb * u_GlobalAmbientColor.rgb;
        float3 N = normalize(In.Normal.xyz);
        float3 V = normalize(u_ViewMatrixInverse[3].xyz - In.WorldPos.xyz);
		
		[unroll]
		for(int i = 0; i < LIGHTCOUNT; ++i)
		{
			if(u_ActiveLights[i])
			{
				float3 L = normalize(u_LightPositions[i].xyz - In.WorldPos.xyz * u_LightPositions[i].w);
				float3 H = normalize(L + V);
				
				float LN = dot(L, N);
				float HN = dot(H, N);
				
				float diff_factor = LN > 0.0f ? LN : 0.0f;
				float spec_factor = (LN > 0.0f && HN > 0.0f) ? pow(HN, matShininess) : 0.0f;
                
				color += matDiffuseColor.rgb * u_LightDiffuseColors[i].rgb * diff_factor;
				color += matSpecularColor.rgb * u_LightSpecularColors[i].rgb * spec_factor;
            }
        }
		
		color = saturate(color);
		result = float4(color, matDiffuseColor.a);
    }
	else
	{
		if(u_VertexColorEnabled)
		{
			result = In.Color;
		}
		else
		{
			result = u_MaterialAmbientColor;
		}
	}
	
	if(u_MaterialHasDiffuseTexture)
	{
		result *= u_MaterialDiffuseTexture.Sample(u_MaterialDiffuseTextureSampler, In.TexCoord);
	}
	
	if(u_AlphaTestEnabled && result.a < u_AlphaTestThreshold)
	{
		discard;
	}
		
	return result;
}
