// Transformation matrices
float4x4  u_WorldViewProjectionMatrix;
float3x3  u_WorldMatrixInverseTranspose;
float4x3  u_WorldMatrix;
float4x3  u_ViewMatrixInverse;

// Light attributes
float     u_LightingEnabled;
float4    u_GlobalAmbientColor;
float     u_ActiveLights[8];
float4    u_LightPositions[8];
float4    u_LightDiffuseColors[8];
float4    u_LightSpecularColors[8];

// Material attributes
float     u_VertexColorEnabled;
float4    u_MaterialAmbientColor;
float4    u_MaterialDiffuseColor;
float4    u_MaterialSpecularColor;
float     u_MaterialShininess;
float     u_MaterialHasDiffuseTexture;
sampler2D u_MaterialDiffuseTexture;
float4x3  u_TextureTransformMatrix;

// Attributes
struct VS_IN
{
    float4 Position : POSITION;
    float3 Normal   : NORMAL;
    float4 Color    : COLOR;
    float2 TexCoord : TEXCOORD0;
};

// Varying parameters
struct VS_OUT
{
    float4 Position : POSITION;
    float4 Color    : COLOR;
    float2 TexCoord : TEXCOORD0;
    float3 Pos      : TEXCOORD1;
    float3 Normal   : TEXCOORD2;
};

VS_OUT vs_main( VS_IN In )
{
    VS_OUT Out = (VS_OUT)0;
    Out.Position = mul( In.Position, u_WorldViewProjectionMatrix );
    Out.Color = In.Color;
    Out.TexCoord = mul( float4(In.TexCoord, 0.0f, 1.0f), u_TextureTransformMatrix ).xy;
    Out.Pos = mul( In.Position, u_WorldMatrix );
    Out.Normal = mul( In.Normal, u_WorldMatrixInverseTranspose );
    return Out;
}

// Varying parameters
struct PS_IN
{
    float4 Color    : COLOR;
    float2 TexCoord : TEXCOORD0;
    float3 Pos      : TEXCOORD1;
    float3 Normal   : TEXCOORD2;
};

float4 ps_main( PS_IN In ) : COLOR
{
	float4 finalColor;
	
    if( u_LightingEnabled )
    {
		float4 matAmbientColor;
		float4 matDiffuseColor;
		float4 matSpecularColor;
		float matShininess;
		
		if( u_VertexColorEnabled )
		{
			matAmbientColor = In.Color;
			matDiffuseColor = In.Color;
			matSpecularColor = u_MaterialSpecularColor;
			matShininess = u_MaterialShininess;
		}
		else
		{
			matAmbientColor = u_MaterialAmbientColor;
			matDiffuseColor = u_MaterialDiffuseColor;
			matSpecularColor = u_MaterialSpecularColor;
			matShininess = u_MaterialShininess;
		}
		
        float3 color = matAmbientColor.rgb * u_GlobalAmbientColor.rgb;
        float3 n = normalize(In.Normal);
        float3 viewVector = normalize( u_ViewMatrixInverse[3] - In.Pos );

		[unroll]
        for( int i = 0; i < 8; i++ )
        {
            if( u_ActiveLights[i] )
            {
                float3 L = normalize( u_LightPositions[i].xyz - In.Pos * u_LightPositions[i].w );
				float3 H = normalize( L + viewVector );
				
				float LN = dot(L, n);
				float HN = dot(H, n);
				
				float diff_factor = LN > 0.0 ? LN : 0.0;
				float spec_factor = (LN > 0.0 && HN > 0.0) ? pow(HN, matShininess) : 0.0;
				
				color += matDiffuseColor.rgb * u_LightDiffuseColors[i].rgb * diff_factor;
				color += matSpecularColor.rgb * u_LightSpecularColors[i].rgb * spec_factor;
            }
            else
                break;
        }
		
		color = saturate(color);
		finalColor = float4(color, matDiffuseColor.a);
    }
	else
	{
		if( u_VertexColorEnabled )
		{
			finalColor = In.Color;
		}
		else
		{
			finalColor = u_MaterialAmbientColor;
		}
	}
	
    if( u_MaterialHasDiffuseTexture )
		finalColor *= tex2D( u_MaterialDiffuseTexture, In.TexCoord );
	
	return finalColor;
}
